/*
 *  This file is part of "Ménapien !",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2008 Olivier Jacquemin.
 *
 *  "Ménapien !" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Ménapien !" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

function QuestionsGenerator() {
    this.parameters = {};
    this.questions = [];
}

/**
 * Abstract: should be overridden.
 */
QuestionsGenerator.prototype.maxDifferentQuestions = -1;

function QuestionsGenerator_generateQuestions(total) {
    var newQuestion;

    if (total < 0 || total > this.maxDifferentQuestions) {
        throw "Unable to generate " + total + " questions: max is " + this.maxDifferentQuestions;
    }

    for (var i = 0; i < total; i += 1) {
        do {
            newQuestion = this.generateNewQuestion();
        } while (this.questionExists(newQuestion));
        this.questions[i] = newQuestion;
    }

    return this.questions;
}

QuestionsGenerator.prototype.generateQuestions = QuestionsGenerator_generateQuestions;

/**
 * Abstract: should be overridden.
 */
function QuestionsGenerator_generateNewQuestion() {
}

QuestionsGenerator.prototype.generateNewQuestion = QuestionsGenerator_generateNewQuestion;

function QuestionsGenerator_questionExists(question) {
    var existingQuestion = null;

    for (var i = 0; i < this.questions.length; i += 1) {
        existingQuestion = this.questions[i];
        if (question.equals(existingQuestion)) {
            return true;
        }
    }

    return false;
}

QuestionsGenerator.prototype.questionExists = QuestionsGenerator_questionExists;

function QuestionsGenerator_allParametersSet() {
    var parameter;
    
    for (var i in this.parameters) {
        parameter = this.parameters[i];
        if ((parameter.value !== 0) && (!parameter.value)) {
            return false;
        }
    }

    return true;
}

QuestionsGenerator.prototype.allParametersSet = QuestionsGenerator_allParametersSet;
