/*
 *  This file is part of "Ménapien !",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2008 Olivier Jacquemin.
 *
 *  "Ménapien !" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Ménapien !" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

/*extern Tester, Multiplication, Addition, Subtraction, IntegerOperationsGenerator, $ */

var tester = null;

function clearAnswer() {
    var element = document.getElementById("answer");
    element.value = "";
    element = document.getElementById("correctionDiv");
    element.className = "invisible";
}

function clearCounter(id) {
    var counterSpan = document.getElementById(id);
    counterSpan.firstChild.data = 0;
}

function clearCounters() {
    clearCounter("correct");
    clearCounter("total");
    clearCounter("remainingCount");
    clearCounter("inCorrectCount");
    clearCounter("correctCount");
}

function hideTestButtons() {
    document.getElementById("testButtonsDiv").className = "hidden";
}

function showTestButtons() {
    document.getElementById("testButtonsDiv").className = "buttonsContainer";
}

function showParameters() {
    document.getElementById("allParams").className = "frame";
    document.getElementById("testDiv").className = "hidden";
    document.getElementById("resultDiv").className = "hidden";
}

function showTest() {
    document.getElementById("allParams").className = "hidden";
    document.getElementById("testDiv").className = "frame";
    document.getElementById("resultDiv").className = "hidden";
}

function showResult() {
    document.getElementById("allParams").className = "hidden";
    document.getElementById("testDiv").className = "hidden";
    document.getElementById("resultDiv").className = "frame";
}

function incrementCounter(id) {
    var counterSpan = document.getElementById(id);
    counterSpan.firstChild.data = (counterSpan.firstChild.data - 0) + 1;
}

function decrementCounter(id) {
    var counterSpan = document.getElementById(id);
    counterSpan.firstChild.data = (counterSpan.firstChild.data - 0) - 1;
}

function restartButton_Click() {
    showParameters();
}

function resultButton_Click() {
    showResult();
}

function askNextQuestion() {
    clearAnswer();

    var element = document.getElementById("question");
    element.firstChild.data = tester.nextQuestion();
    
    $("#remainingCountDiv").effect("transfer",
                                   { to: "#question" },
                                   1000,
                                   function () {				   
                                       decrementCounter("remainingCount");
                                   });

    element = document.getElementById("answer");
    element.disabled = false;
    element.focus();
}

function body_Load() {
    showParameters();
    clearAnswer();
    clearCounters();
}

function startButton_Click() {
    var title = "";
    var questionsGenerator = null;

    showTest();
    hideTestButtons();

    var tableTypeElement = document.getElementById("multiplicationOption");
    if (tableTypeElement.checked) {
        questionsGenerator = new IntegerOperationsGenerator(Multiplication);
        title = "Multiplications : table de ";
    }
    else {
        tableTypeElement = document.getElementById("additionOption");
        if (tableTypeElement.checked) {
            questionsGenerator = new IntegerOperationsGenerator(Addition);
            title = "Additions : table de ";
        }
        else {
            questionsGenerator = new IntegerOperationsGenerator(Subtraction);
            title = "Soustractions : table de ";
        }
    }

    var table = document.getElementById("table").value;
    questionsGenerator.parameters.table.setValue(table);

    title += table;
    document.getElementById("testTitle").firstChild.data = title;

    tester = new Tester(questionsGenerator);
    
    clearCounters();
    document.getElementById("remainingCount").firstChild.data =
        questionsGenerator.maxDifferentQuestions;

    var answerInput = document.getElementById("answer");
    answerInput.disabled = false;
    answerInput.focus();

    askNextQuestion();
}

function fillCell(cell, value) {
    cell.firstChild.data = value;
}

function buildReview() {
    var rows;
    var row;
    var question;
    var duration;
    var maxDuration;
    var faults = 0;
    var score;

    maxDuration = document.getElementById("max").value;
    rows = document.getElementsByTagName("tr");
    for (var i = 0; i < tester.questions.length; i += 1) {
        row = rows[i + 1];
        question = tester.questions[i];
        fillCell(row.childNodes[0], question.getStatement());
        fillCell(row.childNodes[1], question.answeredValue);
        fillCell(row.childNodes[2], question.getResult());

        duration = question.getDuration() / 1000;
        duration = (Math.round(duration * 10) / 10);
        fillCell(row.childNodes[3],
                 duration + " seconde" +
                 (duration > 1 ? "s" : ""));

        row.className = "";
        if (duration > maxDuration) {
            row.className = "late";
        }

        if (!question.isCorrect()) {
            row.className = "wrong";
            faults += 1;
        }
    }

    score = document.getElementById("score");
    switch (faults) {
    case 0:
        score.className = "good";
        break;
    case 1:
        score.className = "bad";
        break;
    default:
        score.className = "ugly";
        break;
    }
}

function doAfterTimeout(funct, milliseconds) {
    if (milliseconds === 0) {
        funct();
    }
    else {
        setTimeout(funct, milliseconds);
    }
}

function finishTest() {
    document.getElementById("question").firstChild.data = "";
    clearAnswer();
    buildReview();
    showTestButtons();
}

function keyPressHandler(event) {
    var answer = document.getElementById("answer");
    var yesNo = document.getElementById("yesNo");
    var correctionDiv = document.getElementById("correctionDiv");
    var correctAnswer = document.getElementById("correctAnswer");

    if (event.keyCode === 13) { // Enter key pressed.
        answer.disabled = true;

        if (tester.answerCurrent(answer.value)) {
            yesNo.firstChild.data = "Oui !";
            correctionDiv.className = "correct";
            correctAnswer.className = "hidden";
            correctAnswer.firstChild.data = "";
            incrementCounter("correct");
            $("#answer").effect("transfer",
                                { to: "#correctCountDiv" },
                                1000,
                                function () {
                                    incrementCounter("correctCount");
                                });
        }
        else {
            yesNo.firstChild.data = "Non !";
            correctionDiv.className = "wrong";
            correctAnswer.className = "";
            correctAnswer.firstChild.data =
                tester.getCurrentQuestionCorrection();
            $("#answer").effect("transfer",
                                { to: "#inCorrectCountDiv" },
                                1000,
                                function () {
                                    incrementCounter("inCorrectCount");
                                });
        }
        incrementCounter("total");
        
        var delay = document.getElementById("delay").value;
        delay = delay - 0;
        if (tester.lastReached()) {
            doAfterTimeout(finishTest, delay * 1000);
        }
        else {
            doAfterTimeout(askNextQuestion, delay * 1000);
        }

        return false;
    }
}
