/*
 *  This file is part of "Ménapien !",
 *  a web-based self-learning applications framework.
 *
 *  Copyright (C) 2007-2008 Olivier Jacquemin.
 *
 *  "Ménapien !" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  "Ménapien !" is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program: see the file COPYING.txt.  If not, see
 *  http://www.gnu.org/licenses/.
 */

/*extern
  assert, assertEquals, assertNotEquals, assertNotUndefined, assertNotNull, assertFalse,
  debug */

/**
 * WARNING: tests in this page must be executed in the specified order.
 */

/**
 * @returns The names of all test functions in this page.
 */
function exposeTestFunctionNames() {
    return [
        "testGetTestDocument",
        "testApplicationParams",
        "testFrameworkParams",
        "testFramesAfterLoad",
        "testFramesAfterStart",
        "testFramesAfterShowResults"
    ];
}

function getTestDocument() {
    return frames[0].document;
}

function simulateEnterKeyPress() {
    var d = getTestDocument();
    var answerInput = d.getElementById("answer");
    var e = null;
    if (window.KeyEvent) {
        // Mozilla-specific.
        e = d.createEvent("KeyEvents");
        e.initKeyEvent("keypress", true, true, null, false, false, false, false, 13, 0);
        answerInput.dispatchEvent(e);
    }
    else {
        // IE-specific.
        e = d.createEventObject();
        e.keyCode = 13;
        answerInput.fireEvent("onkeypress", e);
    }
}

/**
 * Checks the presence of an element in the document under test.
 */
function getTestDocumentElement(id) {
    var doc = getTestDocument();
    var element = doc.getElementById(id);

    assertNotNull("No element with id '" + id + "' found.", element);

    return element;
}

function getElementNextSiblingTag(element) {
    var sibling = element.nextSibling;
    while (sibling && !sibling.tagName) {
        // sibling exists but is not a tag.
        sibling = sibling.nextSibling;
    }
    assertNotNull(sibling);
    return sibling;
}

function getElementAttribute(element, attribute) {
    var browserSpecificAttribute = attribute;
    switch (attribute) {
    case "class":
        if (element.getAttribute("className")) {
            browserSpecificAttribute = "className";
        }
        break;
    case "for":
        if (element.getAttribute("htmlFor")) {
            browserSpecificAttribute = "htmlFor";
        }
        break;
    }
    return element.getAttribute(browserSpecificAttribute);
}

/**
 * Checks the properties of an element.
 *
 * @param element: the element to check.
 * @param tagName: the expected tag name for this element.
 * @param attributeName: name of an attribute to check.
 * @param attributeValue: expected value for this attribute.
 *
 * The last 2 parameters can be repeated as needed to check multiple attributes.
 */
function checkElement(element, tagName, attributeName, attributeValue) {
    var name;
    var value;

    assertNotNull(element);

    assertEquals("Unexpected tag name for element '" + element.id + "'.",
                 tagName.toLowerCase(), element.tagName.toLowerCase());

    assert("Invalid number of arguments (should be even).",
           (arguments.length % 2 === 0));

    for (var i = 2; i < arguments.length; i += 2) {
        attributeName = arguments[i];
        attributeValue = arguments[i + 1];
        assertEquals("Unexpected '" + attributeName +
                     "' attribute for element '" + element.id + "'.",
                     attributeValue, getElementAttribute(element, attributeName));
    }
}

function testGetTestDocument() {
    var d = getTestDocument();
    assertNotUndefined("Invalid object returned by getTestDocument.",
                       d.getElementById);
}

function testApplicationParams() {
    var element;

    element = getTestDocumentElement("multiplicationOption");
    checkElement(element, "input",
                 "type", "radio",
                 "name", "tableType");
    element = getElementNextSiblingTag(element);
    checkElement(element, "label",
                 "for", "multiplicationOption");

    element = getTestDocumentElement("additionOption");
    checkElement(element, "input",
                 "type", "radio",
                 "name", "tableType");
    element = getElementNextSiblingTag(element);
    checkElement(element, "label",
                 "for", "additionOption");

    element = getTestDocumentElement("subtractionOption");
    checkElement(element, "input",
                 "type", "radio",
                 "name", "tableType");
    element = getElementNextSiblingTag(element);
    checkElement(element, "label",
                 "for", "subtractionOption");

    element = getTestDocumentElement("table");
    checkElement(element, "input",
                 "type", "text");
}

function testFrameworkParams() {
    var element;

    element = getTestDocumentElement("max");
    checkElement(element, "input",
                 "type", "text");

    element = getTestDocumentElement("delay");
    checkElement(element, "input",
                 "type", "text");
}

/**
 * Test 1.
 */
function testFramesAfterLoad() {
    var d = getTestDocument();
    
    assertEquals("Incorrect class of allParams frame after document load.",
                 "frame", d.getElementById("allParams").className);
    assertEquals("Incorrect class of testDiv frame after document load.",
                 "hidden", d.getElementById("testDiv").className);
    assertEquals("Incorrect class of resultDiv frame after document load.",
                 "hidden", d.getElementById("resultDiv").className);
}

/**
 * Test 2.
 */
function testFramesAfterStart() {
    var d = getTestDocument();
    d.getElementById("table").value = "3";
    d.getElementById("delay").value = "0";
    d.getElementById("start").onclick();

    assertEquals("Incorrect class of allParams frame after document load.",
                 "hidden", d.getElementById("allParams").className);
    assertEquals("Incorrect class of testDiv frame after document load.",
                 "frame", d.getElementById("testDiv").className);
    assertEquals("Incorrect class of resultDiv frame after document load.",
                 "hidden", d.getElementById("resultDiv").className);
}

/**
 * Test 3.
 */
function testFramesAfterShowResults() {
    var d = getTestDocument();
    d.getElementById("showResults").onclick();

    assertEquals("Incorrect class of allParams frame after document load.",
                 "hidden", d.getElementById("allParams").className);
    assertEquals("Incorrect class of testDiv frame after document load.",
                 "hidden", d.getElementById("testDiv").className);
    assertEquals("Incorrect class of resultDiv frame after document load.",
                 "frame", d.getElementById("resultDiv").className);
}
